@@ -10,9 +10,9 @@
 
 #include <fnmatch.h>
 
-#include "xlator.h"
-#include "defaults.h"
-#include "libglusterfs-messages.h"
+#include "glusterfs/xlator.h"
+#include "glusterfs/defaults.h"
+#include "glusterfs/libglusterfs-messages.h"
 
 #define GF_OPTION_LIST_EMPTY(_opt) (_opt->value[0] == NULL)
 
@@ -25,7 +25,8 @@ xlator_option_validate_path(xlator_t *xl, const char *key, const char *value,
 
     if (strstr(value, "../")) {
         snprintf(errstr, 256, "invalid path given '%s'", value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -35,7 +36,8 @@ xlator_option_validate_path(xlator_t *xl, const char *key, const char *value,
                  "option %s %s: '%s' is not an "
                  "absolute path name",
                  key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -59,7 +61,8 @@ xlator_option_validate_int(xlator_t *xl, const char *key, const char *value,
     if (gf_string2longlong(value, &inputll) != 0) {
         snprintf(errstr, 256, "invalid number format \"%s\" in option \"%s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -67,7 +70,8 @@ xlator_option_validate_int(xlator_t *xl, const char *key, const char *value,
     if ((inputll == 0) && (gf_string2ulonglong(value, &uinputll) != 0)) {
         snprintf(errstr, 256, "invalid number format \"%s\" in option \"%s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -87,8 +91,8 @@ xlator_option_validate_int(xlator_t *xl, const char *key, const char *value,
                      "'%lld' in 'option %s %s' is smaller than "
                      "minimum value '%.0f'",
                      inputll, key, value, opt->min);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                    errstr, NULL);
             goto out;
         }
     } else if (opt->validate == GF_OPT_VALIDATE_MAX) {
@@ -97,8 +101,8 @@ xlator_option_validate_int(xlator_t *xl, const char *key, const char *value,
                      "'%lld' in 'option %s %s' is greater than "
                      "maximum value '%.0f'",
                      inputll, key, value, opt->max);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                    errstr, NULL);
             goto out;
         }
     } else if ((inputll < opt->min) || (inputll > opt->max)) {
@@ -106,7 +110,8 @@ xlator_option_validate_int(xlator_t *xl, const char *key, const char *value,
                  "'%lld' in 'option %s %s' is out of range "
                  "[%.0f - %.0f]",
                  inputll, key, value, opt->min, opt->max);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -121,15 +126,16 @@ static int
 xlator_option_validate_sizet(xlator_t *xl, const char *key, const char *value,
                              volume_option_t *opt, char **op_errstr)
 {
-    size_t size = 0;
+    uint64_t size = 0;
     int ret = 0;
     char errstr[256];
 
     /* Check the range */
-    if (gf_string2bytesize_size(value, &size) != 0) {
+    if (gf_string2bytesize_uint64(value, &size) != 0) {
         snprintf(errstr, 256, "invalid number format \"%s\" in option \"%s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         ret = -1;
         goto out;
     }
@@ -144,11 +150,11 @@ xlator_option_validate_sizet(xlator_t *xl, const char *key, const char *value,
 
     if ((size < opt->min) || (size > opt->max)) {
         snprintf(errstr, 256,
-                 "'%" GF_PRI_SIZET
-                 "' in 'option %s %s' "
-                 "is out of range [%.0f - %.0f]",
+                 "'%" PRIu64
+                 "' in 'option %s %s' is out of range [%.0f - %.0f]",
                  size, key, value, opt->min, opt->max);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                errstr, NULL);
         ret = -1;
     }
 
@@ -172,7 +178,8 @@ xlator_option_validate_bool(xlator_t *xl, const char *key, const char *value,
     if (gf_string2boolean(value, &is_valid) != 0) {
         snprintf(errstr, 256, "option %s %s: '%s' is not a valid boolean value",
                  key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -207,7 +214,8 @@ xlator_option_validate_xlator(xlator_t *xl, const char *key, const char *value,
     if (!xlopt) {
         snprintf(errstr, 256, "option %s %s: '%s' is not a valid volume name",
                  key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -218,7 +226,7 @@ out:
     return ret;
 }
 
-void
+static void
 set_error_str(char *errstr, size_t len, volume_option_t *opt, const char *key,
               const char *value)
 {
@@ -240,18 +248,15 @@ set_error_str(char *errstr, size_t len, volume_option_t *opt, const char *key,
     return;
 }
 
-int
+static int
 is_all_whitespaces(const char *value)
 {
     int i = 0;
-    size_t len = 0;
 
     if (value == NULL)
         return -1;
 
-    len = strlen(value);
-
-    for (i = 0; i < len; i++) {
+    for (i = 0; value[i] != '\0'; i++) {
         if (value[i] == ' ')
             continue;
         else
@@ -267,9 +272,6 @@ xlator_option_validate_str(xlator_t *xl, const char *key, const char *value,
 {
     int ret = -1;
     int i = 0;
-    char errstr[4096] = {
-        0,
-    };
 
     /* Check if the '*str' is valid */
     if (GF_OPTION_LIST_EMPTY(opt)) {
@@ -309,9 +311,11 @@ xlator_option_validate_str(xlator_t *xl, const char *key, const char *value,
 
 out:
     if (ret) {
+        char errstr[4096];
         set_error_str(errstr, sizeof(errstr), opt, key, value);
 
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         if (op_errstr)
             *op_errstr = gf_strdup(errstr);
     }
@@ -330,7 +334,8 @@ xlator_option_validate_percent(xlator_t *xl, const char *key, const char *value,
     if (gf_string2percent(value, &percent) != 0) {
         snprintf(errstr, 256, "invalid percent format \"%s\" in \"option %s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -338,7 +343,8 @@ xlator_option_validate_percent(xlator_t *xl, const char *key, const char *value,
         snprintf(errstr, 256,
                  "'%lf' in 'option %s %s' is out of range [0 - 100]", percent,
                  key, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -384,8 +390,8 @@ xlator_option_validate_percent_or_sizet(xlator_t *xl, const char *key,
                          "'%lf' in 'option %s %s' is out"
                          " of range [0 - 100]",
                          size, key, value);
-                gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
-                       errstr);
+                gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE,
+                        "error=%s", errstr, NULL);
                 goto out;
             }
             ret = 0;
@@ -400,8 +406,8 @@ xlator_option_validate_percent_or_sizet(xlator_t *xl, const char *key,
                      " %s' should not be fractional value. Use "
                      "valid unsigned integer value.",
                      size, key, value);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                    errstr, NULL);
             goto out;
         }
 
@@ -419,8 +425,8 @@ xlator_option_validate_percent_or_sizet(xlator_t *xl, const char *key,
                      "'%lf' in 'option %s %s'"
                      " is out of range [%.0f - %.0f]",
                      size, key, value, opt->min, opt->max);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                    errstr, NULL);
             goto out;
         }
         ret = 0;
@@ -431,7 +437,8 @@ xlator_option_validate_percent_or_sizet(xlator_t *xl, const char *key,
 
     snprintf(errstr, 256, "invalid number format \"%s\" in \"option %s\"",
              value, key);
-    gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+    gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s", errstr,
+            NULL);
 
 out:
     if (ret && op_errstr)
@@ -453,7 +460,8 @@ xlator_option_validate_time(xlator_t *xl, const char *key, const char *value,
                  "invalid time format \"%s\" in "
                  "\"option %s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -472,7 +480,8 @@ xlator_option_validate_time(xlator_t *xl, const char *key, const char *value,
                  "' in 'option %s %s' is "
                  "out of range [%.0f - %.0f]",
                  input_time, key, value, opt->min, opt->max);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -495,7 +504,8 @@ xlator_option_validate_double(xlator_t *xl, const char *key, const char *value,
     if (gf_string2double(value, &input) != 0) {
         snprintf(errstr, 256, "invalid number format \"%s\" in option \"%s\"",
                  value, key);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -515,8 +525,8 @@ xlator_option_validate_double(xlator_t *xl, const char *key, const char *value,
                      "'%f' in 'option %s %s' is smaller than "
                      "minimum value '%f'",
                      input, key, value, opt->min);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                    errstr, NULL);
             goto out;
         }
     } else if (opt->validate == GF_OPT_VALIDATE_MAX) {
@@ -525,8 +535,8 @@ xlator_option_validate_double(xlator_t *xl, const char *key, const char *value,
                      "'%f' in 'option %s %s' is greater than "
                      "maximum value '%f'",
                      input, key, value, opt->max);
-            gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s",
-                   errstr);
+            gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                    errstr, NULL);
             goto out;
         }
     } else if ((input < opt->min) || (input > opt->max)) {
@@ -534,7 +544,8 @@ xlator_option_validate_double(xlator_t *xl, const char *key, const char *value,
                  "'%f' in 'option %s %s' is out of range "
                  "[%f - %f]",
                  input, key, value, opt->min, opt->max);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_OUT_OF_RANGE, "error=%s",
+                errstr, NULL);
         goto out;
     }
 
@@ -552,12 +563,11 @@ xlator_option_validate_addr(xlator_t *xl, const char *key, const char *value,
     int ret = -1;
     char errstr[256];
 
-    if (!valid_internet_address((char *)value, _gf_false)) {
-        snprintf(errstr, 256,
-                 "option %s %s: '%s'  is not a valid internet-address,"
-                 " it does not conform to standards.",
-                 key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+    if (!valid_internet_address((char *)value, _gf_false, _gf_false)) {
+        snprintf(errstr, 256, "option %s %s: Can not parse %s address", key,
+                 value, value);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         if (op_errstr)
             *op_errstr = gf_strdup(errstr);
     }
@@ -583,9 +593,6 @@ xlator_option_validate_addr_list(xlator_t *xl, const char *key,
     char *addr_list = NULL;
     char *addr = NULL;
     char *dir = NULL;
-    char errstr[4096] = {
-        0,
-    };
 
     dup_val = gf_strdup(value);
     if (!dup_val)
@@ -595,7 +602,7 @@ xlator_option_validate_addr_list(xlator_t *xl, const char *key,
         /* Possible old format, handle it for back-ward compatibility */
         addr_tok = strtok_r(dup_val, ",", &save_ptr);
         while (addr_tok) {
-            if (!valid_internet_address(addr_tok, _gf_true))
+            if (!valid_internet_address(addr_tok, _gf_true, _gf_true))
                 goto out;
 
             addr_tok = strtok_r(NULL, ",", &save_ptr);
@@ -630,7 +637,7 @@ xlator_option_validate_addr_list(xlator_t *xl, const char *key,
         if (addr_tok == NULL)
             goto out;
         while (addr_tok) {
-            if (!valid_internet_address(addr_tok, _gf_true))
+            if (!valid_internet_address(addr_tok, _gf_true, _gf_true))
                 goto out;
 
             addr_tok = strtok_r(NULL, "|", &save_ptr);
@@ -646,11 +653,13 @@ xlator_option_validate_addr_list(xlator_t *xl, const char *key,
 
 out:
     if (ret) {
+        char errstr[4096];
         snprintf(errstr, sizeof(errstr),
                  "option %s %s: '%s' is not "
                  "a valid internet-address-list",
                  key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         if (op_errstr)
             *op_errstr = gf_strdup(errstr);
     }
@@ -668,9 +677,6 @@ xlator_option_validate_mntauth(xlator_t *xl, const char *key, const char *value,
     char *dup_val = NULL;
     char *addr_tok = NULL;
     char *save_ptr = NULL;
-    char errstr[4096] = {
-        0,
-    };
 
     dup_val = gf_strdup(value);
     if (!dup_val)
@@ -689,11 +695,13 @@ xlator_option_validate_mntauth(xlator_t *xl, const char *key, const char *value,
 
 out:
     if (ret) {
+        char errstr[4096];
         snprintf(errstr, sizeof(errstr),
                  "option %s %s: '%s' is not "
                  "a valid mount-auth-address",
                  key, value, value);
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "%s", errstr);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY, "error=%s",
+                errstr, NULL);
         if (op_errstr)
             *op_errstr = gf_strdup(errstr);
     }
@@ -706,12 +714,12 @@ out:
 static int
 gf_validate_size(const char *sizestr, volume_option_t *opt)
 {
-    size_t value = 0;
+    uint64_t value = 0;
     int ret = 0;
 
     GF_ASSERT(opt);
 
-    if (gf_string2bytesize_size(sizestr, &value) != 0 || value < opt->min ||
+    if (gf_string2bytesize_uint64(sizestr, &value) != 0 || value < opt->min ||
         value % 512) {
         ret = -1;
         goto out;
@@ -759,20 +767,16 @@ validate_list_elements(const char *string, volume_option_t *opt,
         key = strtok_r(str_ptr, ":", &substr_sav);
         if (!key || (key_validator && key_validator(key))) {
             ret = -1;
-            gf_msg(THIS->name, GF_LOG_WARNING, 0, LG_MSG_INVALID_ENTRY,
-                   "invalid list '%s', key "
-                   "'%s' not valid.",
-                   string, key);
+            gf_smsg(THIS->name, GF_LOG_WARNING, 0, LG_MSG_INVALID_ENTRY,
+                    "list=%s", string, "key=%s", key ? key : "", NULL);
             goto out;
         }
 
         value = strtok_r(NULL, ":", &substr_sav);
         if (!value || (value_validator && value_validator(value, opt))) {
             ret = -1;
-            gf_msg(THIS->name, GF_LOG_WARNING, 0, LG_MSG_INVALID_ENTRY,
-                   "invalid list '%s', "
-                   "value '%s' not valid.",
-                   string, key);
+            gf_smsg(THIS->name, GF_LOG_WARNING, 0, LG_MSG_INVALID_ENTRY,
+                    "list=%s", string, "value=%s", key, NULL);
             goto out;
         }
 
@@ -877,8 +881,8 @@ xlator_option_validate(xlator_t *xl, char *key, char *value,
     };
 
     if (opt->type > GF_OPTION_TYPE_MAX) {
-        gf_msg(xl->name, GF_LOG_ERROR, 0, LG_MSG_INVALID_ENTRY,
-               "unknown option type '%d'", opt->type);
+        gf_smsg(xl->name, GF_LOG_ERROR, 0, LG_MSG_UNKNOWN_OPTION_TYPE,
+                "type=%d", opt->type, NULL);
         goto out;
     }
 
@@ -894,7 +898,6 @@ xlator_volume_option_get_list(volume_opt_list_t *vol_list, const char *key)
 {
     volume_option_t *opt = NULL;
     volume_opt_list_t *opt_list = NULL;
-    volume_option_t *found = NULL;
     int index = 0;
     int i = 0;
     char *cmp_key = NULL;
@@ -911,13 +914,12 @@ xlator_volume_option_get_list(volume_opt_list_t *vol_list, const char *key)
             if (!cmp_key)
                 break;
             if (fnmatch(cmp_key, key, FNM_NOESCAPE) == 0) {
-                found = &opt[index];
-                goto out;
+                return &opt[index];
             }
         }
     }
-out:
-    return found;
+
+    return NULL;
 }
 
 volume_option_t *
@@ -961,18 +963,16 @@ xl_opt_validate(dict_t *dict, char *key, data_t *value, void *data)
 
     ret = xlator_option_validate(xl, key, value->data, opt, &errstr);
     if (ret)
-        gf_msg(xl->name, GF_LOG_WARNING, 0, LG_MSG_VALIDATE_RETURNS,
-               "validate of %s returned %d", key, ret);
+        gf_smsg(xl->name, GF_LOG_WARNING, 0, LG_MSG_VALIDATE_RETURNS, "key=%s",
+                key, "ret=%d", ret, NULL);
 
     if (errstr)
         /* possible small leak of previously set stub->errstr */
         stub->errstr = errstr;
 
     if (fnmatch(opt->key[0], key, FNM_NOESCAPE) != 0) {
-        gf_msg(xl->name, GF_LOG_DEBUG, 0, LG_MSG_INVALID_ENTRY,
-               "option '%s' is deprecated, preferred is '%s', "
-               "continuing with correction",
-               key, opt->key[0]);
+        gf_smsg(xl->name, GF_LOG_DEBUG, 0, LG_MSG_OPTION_DEPRECATED, "key=%s",
+                key, "preferred=%s", opt->key[0], NULL);
         dict_set(dict, opt->key[0], value);
         dict_del(dict, key);
     }
@@ -1040,9 +1040,8 @@ xlator_validate_rec(xlator_t *xlator, char **op_errstr)
 
     while (trav) {
         if (xlator_validate_rec(trav->xlator, op_errstr)) {
-            gf_msg("xlator", GF_LOG_WARNING, 0, LG_MSG_VALIDATE_REC_FAILED,
-                   "validate_rec "
-                   "failed");
+            gf_smsg("xlator", GF_LOG_WARNING, 0, LG_MSG_VALIDATE_REC_FAILED,
+                    NULL);
             goto out;
         }
 
@@ -1066,8 +1065,8 @@ xlator_validate_rec(xlator_t *xlator, char **op_errstr)
     THIS = old_THIS;
 
     if (ret) {
-        gf_msg(xlator->name, GF_LOG_INFO, 0, LG_MSG_INVALID_ENTRY, "%s",
-               *op_errstr);
+        gf_smsg(xlator->name, GF_LOG_INFO, 0, LG_MSG_INVALID_ENTRY, "%s",
+                *op_errstr, NULL);
         goto out;
     }
 
@@ -1123,6 +1122,7 @@ xlator_reconfigure_rec(xlator_t *old_xl, xlator_t *new_xl)
         THIS = old_xl;
 
         xlator_init_lock();
+        handle_default_options(old_xl, new_xl->options);
         ret = old_xl->reconfigure(old_xl, new_xl->options);
         xlator_init_unlock();
 
@@ -1200,18 +1200,18 @@ pc_or_size(char *in, double *out)
 {
     double pc = 0;
     int ret = 0;
-    size_t size = 0;
+    uint64_t size = 0;
 
     if (gf_string2percent(in, &pc) == 0) {
         if (pc > 100.0) {
-            ret = gf_string2bytesize_size(in, &size);
+            ret = gf_string2bytesize_uint64(in, &size);
             if (!ret)
                 *out = size;
         } else {
             *out = pc;
         }
     } else {
-        ret = gf_string2bytesize_size(in, &size);
+        ret = gf_string2bytesize_uint64(in, &size);
         if (!ret)
             *out = size;
     }
@@ -1223,7 +1223,7 @@ DEFINE_INIT_OPT(uint64_t, uint64, gf_string2uint64);
 DEFINE_INIT_OPT(int64_t, int64, gf_string2int64);
 DEFINE_INIT_OPT(uint32_t, uint32, gf_string2uint32);
 DEFINE_INIT_OPT(int32_t, int32, gf_string2int32);
-DEFINE_INIT_OPT(size_t, size, gf_string2bytesize_size);
+DEFINE_INIT_OPT(uint64_t, size, gf_string2bytesize_uint64);
 DEFINE_INIT_OPT(uint64_t, size_uint64, gf_string2bytesize_uint64);
 DEFINE_INIT_OPT(double, percent, gf_string2percent);
 DEFINE_INIT_OPT(double, percent_or_size, pc_or_size);
@@ -1238,7 +1238,7 @@ DEFINE_RECONF_OPT(uint64_t, uint64, gf_string2uint64);
 DEFINE_RECONF_OPT(int64_t, int64, gf_string2int64);
 DEFINE_RECONF_OPT(uint32_t, uint32, gf_string2uint32);
 DEFINE_RECONF_OPT(int32_t, int32, gf_string2int32);
-DEFINE_RECONF_OPT(size_t, size, gf_string2bytesize_size);
+DEFINE_RECONF_OPT(uint64_t, size, gf_string2bytesize_uint64);
 DEFINE_RECONF_OPT(uint64_t, size_uint64, gf_string2bytesize_uint64);
 DEFINE_RECONF_OPT(double, percent, gf_string2percent);
 DEFINE_RECONF_OPT(double, percent_or_size, pc_or_size);
