@@ -11,7 +11,7 @@
 #ifndef __CDC_MEM_TYPES_H
 #define __CDC_MEM_TYPES_H
 
-#include "mem-types.h"
+#include <glusterfs/mem-types.h>
 
 enum gf_cdc_mem_types {
     gf_cdc_mt_priv_t = gf_common_mt_end + 1,
