@@ -17,7 +17,9 @@
 #include "glusterd-svc-mgmt.h"
 #include "glusterd-shd-svc.h"
 #include "glusterd-quotad-svc.h"
+#ifdef BUILD_GNFS
 #include "glusterd-nfs-svc.h"
+#endif
 #include "glusterd-bitd-svc.h"
 #include "glusterd-shd-svc-helper.h"
 #include "glusterd-scrub-svc.h"
@@ -38,11 +40,12 @@ glusterd_svcs_reconfigure(glusterd_volinfo_t *volinfo)
     conf = this->private;
     GF_ASSERT(conf);
 
+#ifdef BUILD_GNFS
     svc_name = "nfs";
     ret = glusterd_nfssvc_reconfigure();
     if (ret)
         goto out;
-
+#endif
     svc_name = "self-heald";
     if (volinfo) {
         ret = glusterd_shdsvc_reconfigure(volinfo);
@@ -84,10 +87,11 @@ glusterd_svcs_stop(glusterd_volinfo_t *volinfo)
     priv = this->private;
     GF_ASSERT(priv);
 
+#ifdef BUILD_GNFS
     ret = priv->nfs_svc.stop(&(priv->nfs_svc), SIGKILL);
     if (ret)
         goto out;
-
+#endif
     ret = priv->quotad_svc.stop(&(priv->quotad_svc), SIGTERM);
     if (ret)
         goto out;
@@ -103,6 +107,7 @@ glusterd_svcs_stop(glusterd_volinfo_t *volinfo)
         goto out;
 
     ret = priv->scrub_svc.stop(&(priv->scrub_svc), SIGTERM);
+
 out:
     return ret;
 }
@@ -122,10 +127,11 @@ glusterd_svcs_manager(glusterd_volinfo_t *volinfo)
     if (volinfo && volinfo->is_snap_volume)
         return 0;
 
+#if BUILD_GNFS
     ret = conf->nfs_svc.manager(&(conf->nfs_svc), NULL, PROC_START_NO_WAIT);
     if (ret)
         goto out;
-
+#endif
     if (conf->op_version == GD_OP_VERSION_MIN)
         goto out;
 
@@ -294,7 +300,7 @@ glusterd_volume_svc_check_volfile_identical(
 
     this = THIS;
 
-    GF_VALIDATE_OR_GOTO(this->name, this, out);
+    GF_VALIDATE_OR_GOTO("glusterd", this, out);
     GF_VALIDATE_OR_GOTO(this->name, identical, out);
 
     /* This builds volfile for volume level dameons */
@@ -396,15 +402,38 @@ out:
     return ret;
 }
 
+gf_boolean_t
+glusterd_is_svcproc_attachable(glusterd_svc_proc_t *svc_proc)
+{
+    int pid = -1;
+    glusterd_svc_t *parent_svc = NULL;
+
+    if (!svc_proc)
+        return _gf_false;
+
+    if (svc_proc->status == GF_SVC_STARTING)
+        return _gf_true;
+
+    if (svc_proc->status == GF_SVC_STARTED ||
+        svc_proc->status == GF_SVC_DISCONNECTED) {
+        parent_svc = cds_list_entry(svc_proc->svcs.next, glusterd_svc_t,
+                                    mux_svc);
+        if (parent_svc && gf_is_service_running(parent_svc->proc.pidfile, &pid))
+            return _gf_true;
+    }
+
+    if (svc_proc->status == GF_SVC_DIED || svc_proc->status == GF_SVC_STOPPING)
+        return _gf_false;
+
+    return _gf_false;
+}
+
 void *
 __gf_find_compatible_svc(gd_node_type daemon)
 {
     glusterd_svc_proc_t *svc_proc = NULL;
-    glusterd_svc_proc_t *return_proc = NULL;
-    glusterd_svc_t *parent_svc = NULL;
     struct cds_list_head *svc_procs = NULL;
     glusterd_conf_t *conf = NULL;
-    int pid = -1;
 
     conf = THIS->private;
     GF_VALIDATE_OR_GOTO("glusterd", conf, out);
@@ -422,15 +451,7 @@ __gf_find_compatible_svc(gd_node_type daemon)
 
     cds_list_for_each_entry(svc_proc, svc_procs, svc_proc_list)
     {
-        parent_svc = cds_list_entry(svc_proc->svcs.next, glusterd_svc_t,
-                                    mux_svc);
-        if (!return_proc)
-            return_proc = svc_proc;
-
-        /* If there is an  already running shd daemons, select it. Otehrwise
-         * select the first one.
-         */
-        if (parent_svc && gf_is_service_running(parent_svc->proc.pidfile, &pid))
+        if (glusterd_is_svcproc_attachable(svc_proc))
             return (void *)svc_proc;
         /*
          * Logic to select one process goes here. Currently there is only one
@@ -438,7 +459,7 @@ __gf_find_compatible_svc(gd_node_type daemon)
          */
     }
 out:
-    return return_proc;
+    return NULL;
 }
 
 glusterd_svc_proc_t *
@@ -455,6 +476,7 @@ glusterd_svcprocess_new()
     CDS_INIT_LIST_HEAD(&new_svcprocess->svc_proc_list);
     CDS_INIT_LIST_HEAD(&new_svcprocess->svcs);
     new_svcprocess->notify = glusterd_muxsvc_common_rpc_notify;
+    new_svcprocess->status = GF_SVC_STARTING;
     return new_svcprocess;
 }
 
@@ -467,6 +489,7 @@ glusterd_shd_svc_mux_init(glusterd_volinfo_t *volinfo, glusterd_svc_t *svc)
     glusterd_conf_t *conf = NULL;
     glusterd_svc_t *parent_svc = NULL;
     int pid = -1;
+    gf_boolean_t stop_daemon = _gf_false;
     char pidfile[PATH_MAX] = {
         0,
     };
@@ -507,9 +530,10 @@ glusterd_shd_svc_mux_init(glusterd_volinfo_t *volinfo, glusterd_svc_t *svc)
 
             if (!mux_proc) {
                 if (pid != -1 && sys_access(pidfile, R_OK) == 0) {
-                    /* stale pid file, stop and unlink it */
-                    glusterd_proc_stop(&svc->proc, SIGTERM, PROC_STOP_FORCE);
-                    glusterd_unlink_file(pidfile);
+                    /* stale pid file, stop and unlink it. This has to be
+                     * done outside the attach_lock.
+                     */
+                    stop_daemon = _gf_true;
                 }
                 mux_proc = __gf_find_compatible_svc(GD_NODE_SHD);
             }
@@ -517,7 +541,6 @@ glusterd_shd_svc_mux_init(glusterd_volinfo_t *volinfo, glusterd_svc_t *svc)
                 /* Take first entry from the process */
                 parent_svc = cds_list_entry(mux_proc->svcs.next, glusterd_svc_t,
                                             mux_svc);
-                glusterd_copy_file(parent_svc->proc.pidfile, svc->proc.pidfile);
                 mux_conn = &parent_svc->conn;
                 if (volinfo)
                     volinfo->shd.attached = _gf_true;
@@ -548,6 +571,10 @@ glusterd_shd_svc_mux_init(glusterd_volinfo_t *volinfo, glusterd_svc_t *svc)
 unlock:
     pthread_mutex_unlock(&conf->attach_lock);
 out:
+    if (stop_daemon) {
+        glusterd_proc_stop(&svc->proc, SIGTERM, PROC_STOP_FORCE);
+        glusterd_unlink_file(pidfile);
+    }
     return ret;
 }
 
@@ -580,7 +607,8 @@ __gf_find_compatible_svc_from_pid(gd_node_type daemon, pid_t pid)
         cds_list_for_each_entry(svc, &svc_proc->svcs, mux_svc)
         {
             if (gf_is_service_running(svc->proc.pidfile, &mux_pid)) {
-                if (mux_pid == pid) {
+                if (mux_pid == pid &&
+                    glusterd_is_svcproc_attachable(svc_proc)) {
                     /*TODO
                      * inefficient loop, but at the moment, there is only
                      * one shd.
@@ -606,7 +634,9 @@ my_callback(struct rpc_req *req, struct iovec *iov, int count, void *v_frame)
     conf = this->private;
     GF_VALIDATE_OR_GOTO(this->name, conf, out);
 
-    GF_ATOMIC_DEC(conf->blockers);
+    if (GF_ATOMIC_DEC(conf->blockers) == 0) {
+        synccond_broadcast(&conf->cond_blockers);
+    }
 
     STACK_DESTROY(frame->root);
 out:
@@ -694,7 +724,9 @@ out:
     if (volinfo)
         glusterd_volinfo_unref(volinfo);
 
-    GF_ATOMIC_DEC(conf->blockers);
+    if (GF_ATOMIC_DEC(conf->blockers) == 0) {
+        synccond_broadcast(&conf->cond_blockers);
+    }
     STACK_DESTROY(frame->root);
     return 0;
 }
@@ -812,7 +844,6 @@ __glusterd_send_svc_configure_req(glusterd_svc_t *svc, int flags,
                        "to request buffer");
                 goto *errlbl;
             }
-            dict->extra_free = brick_req.dict.dict_val;
         }
 
         frame->cookie = svc;
@@ -874,6 +905,8 @@ maybe_free_iobuf:
 err:
     if (dict)
         dict_unref(dict);
+    if (brick_req.dict.dict_val)
+        GF_FREE(brick_req.dict.dict_val);
 
     GF_FREE(volfile_content);
     if (spec_fd >= 0)
@@ -902,6 +935,22 @@ glusterd_attach_svc(glusterd_svc_t *svc, glusterd_volinfo_t *volinfo, int flags)
 
     rpc = rpc_clnt_ref(svc->conn.rpc);
     for (tries = 15; tries > 0; --tries) {
+        /* There might be a case that the volume for which we're attempting to
+         * attach a shd svc might become stale and in the process of deletion.
+         * Given that the volinfo object is being already passed here before
+         * that sequence of operation has happened we might be operating on a
+         * stale volume. At every sync task switch we should check for existance
+         * of the volume now
+         */
+        if (!glusterd_volume_exists(volinfo->volname)) {
+            gf_msg(THIS->name, GF_LOG_INFO, 0, GD_MSG_SVC_ATTACH_FAIL,
+                   "Volume %s "
+                   " is marked as stale, not attempting further shd svc attach "
+                   "attempts",
+                   volinfo->volname);
+            ret = 0;
+            goto out;
+        }
         if (rpc) {
             pthread_mutex_lock(&conf->attach_lock);
             {
@@ -924,7 +973,7 @@ glusterd_attach_svc(glusterd_svc_t *svc, glusterd_volinfo_t *volinfo, int flags)
          * TBD: see if there's a better way
          */
         synclock_unlock(&conf->big_lock);
-        sleep(1);
+        synctask_sleep(1);
         synclock_lock(&conf->big_lock);
     }
     ret = -1;
@@ -978,7 +1027,7 @@ glusterd_detach_svc(glusterd_svc_t *svc, glusterd_volinfo_t *volinfo, int sig)
          * TBD: see if there's a better way
          */
         synclock_unlock(&conf->big_lock);
-        sleep(1);
+        synctask_sleep(1);
         synclock_lock(&conf->big_lock);
     }
     ret = -1;
