@@ -37,7 +37,9 @@
 #include "glusterd-locks.h"
 #include "glusterd-svc-mgmt.h"
 #include "glusterd-shd-svc.h"
+#ifdef BUILD_GNFS
 #include "glusterd-nfs-svc.h"
+#endif
 #include "glusterd-bitd-svc.h"
 #include "glusterd-scrub-svc.h"
 #include "glusterd-quotad-svc.h"
@@ -555,7 +557,9 @@ glusterd_crt_georep_folders(char *georepdir, glusterd_conf_t *conf)
     char *greplg_s = NULL;
     struct group *gr = NULL;
     int ret = 0;
+    int gr_ret = 0;
     int32_t len = 0;
+    char logdir[PATH_MAX] = {0};
 
     GF_ASSERT(georepdir);
     GF_ASSERT(conf);
@@ -573,61 +577,13 @@ glusterd_crt_georep_folders(char *georepdir, glusterd_conf_t *conf)
         ret = -1;
         goto out;
     }
-    ret = mkdir_p(georepdir, 0777, _gf_true);
+    ret = mkdir_p(georepdir, 0755, _gf_true);
     if (-1 == ret) {
         gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create " GEOREP " directory %s", georepdir);
         goto out;
     }
 
-    if (SLEN(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP) >= PATH_MAX) {
-        ret = -1;
-        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
-               "directory path " DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
-               " is longer than PATH_MAX");
-        goto out;
-    }
-    ret = mkdir_p(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP, 0777, _gf_true);
-    if (-1 == ret) {
-        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
-               "Unable to create " GEOREP " log directory");
-        goto out;
-    }
-
-    /* Slave log file directory */
-    if (SLEN(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP "-slaves") >= PATH_MAX) {
-        ret = -1;
-        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
-               "directory path " DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
-               "-slaves"
-               " is longer than PATH_MAX");
-        goto out;
-    }
-    ret = mkdir_p(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP "-slaves", 0777,
-                  _gf_true);
-    if (-1 == ret) {
-        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
-               "Unable to create " GEOREP " slave log directory");
-        goto out;
-    }
-
-    /* MountBroker log file directory */
-    if (SLEN(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP "-slaves/mbr") >= PATH_MAX) {
-        ret = -1;
-        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
-               "directory path " DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
-               "-slaves/mbr"
-               " is longer than PATH_MAX");
-        goto out;
-    }
-    ret = mkdir_p(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP "-slaves/mbr", 0777,
-                  _gf_true);
-    if (-1 == ret) {
-        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
-               "Unable to create " GEOREP " mountbroker slave log directory");
-        goto out;
-    }
-
     ret = dict_get_str(THIS->options, GEOREP "-log-group", &greplg_s);
     if (ret)
         ret = 0;
@@ -636,21 +592,83 @@ glusterd_crt_georep_folders(char *georepdir, glusterd_conf_t *conf)
         if (!gr) {
             gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_LOGGROUP_INVALID,
                    "group " GEOREP "-log-group %s does not exist", greplg_s);
-            ret = -1;
-            goto out;
+            gr_ret = -1;
         }
-
-        ret = group_write_allow(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP,
-                                gr->gr_gid);
-        if (ret == 0)
-            ret = group_write_allow(
-                DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP "-slaves", gr->gr_gid);
-        if (ret == 0)
-            ret = group_write_allow(DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
-                                                               "-slaves/mbr",
-                                    gr->gr_gid);
+    }
+    if ((strlen(conf->logdir) + 2 + SLEN(GEOREP)) >= PATH_MAX) {
+        ret = -1;
+        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
+               "directory path %s/" GEOREP " is longer than PATH_MAX",
+               conf->logdir);
+        goto out;
+    }
+    len = snprintf(logdir, PATH_MAX, "%s/" GEOREP, conf->logdir);
+    if ((len < 0) || (len >= PATH_MAX)) {
+        ret = -1;
+        goto out;
+    }
+    ret = mkdir_p(logdir, 0755, _gf_true);
+    if (-1 == ret) {
+        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
+               "Unable to create " GEOREP " log directory");
+        goto out;
+    }
+    if (gr) {
+        gr_ret = group_write_allow(logdir, gr->gr_gid);
     }
 
+    if ((strlen(conf->logdir) + 2 + SLEN(GEOREP "-slaves")) >= PATH_MAX) {
+        ret = -1;
+        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
+               "directory path  %s/" GEOREP
+               "-slaves"
+               " is longer than PATH_MAX",
+               conf->logdir);
+        goto out;
+    }
+    len = snprintf(logdir, PATH_MAX, "%s/" GEOREP "-slaves", conf->logdir);
+    if ((len < 0) || (len >= PATH_MAX)) {
+        ret = -1;
+        goto out;
+    }
+    ret = mkdir_p(logdir, 0755, _gf_true);
+    if (-1 == ret) {
+        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
+               "Unable to create " GEOREP " slave log directory");
+        goto out;
+    }
+    if (gr && !gr_ret) {
+        gr_ret = group_write_allow(logdir, gr->gr_gid);
+    }
+
+    /* MountBroker log file directory */
+    if ((strlen(conf->logdir) + 2 + SLEN(GEOREP "-slaves/mbr")) >= PATH_MAX) {
+        ret = -1;
+        gf_msg("glusterd", GF_LOG_CRITICAL, 0, GD_MSG_DIRPATH_TOO_LONG,
+               "directory path  %s/" GEOREP
+               "-slaves/mbr"
+               " is longer than PATH_MAX",
+               conf->logdir);
+        goto out;
+    }
+
+    len = snprintf(logdir, PATH_MAX, "%s/" GEOREP "-slaves/mbr", conf->logdir);
+    if ((len < 0) || (len >= PATH_MAX)) {
+        ret = -1;
+        goto out;
+    }
+
+    ret = mkdir_p(logdir, 0755, _gf_true);
+    if (-1 == ret) {
+        gf_msg("glusterd", GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
+               "Unable to create " GEOREP " mountbroker slave log directory");
+        goto out;
+    }
+    if (gr && !gr_ret) {
+        gr_ret = group_write_allow(logdir, gr->gr_gid);
+    }
+    if (gr_ret)
+        ret = gr_ret;
 out:
     gf_msg_debug("glusterd", 0, "Returning %d", ret);
     return ret;
@@ -814,18 +832,19 @@ configure_syncdaemon(glusterd_conf_t *conf)
 
     /* log-file */
     runinit_gsyncd_setrx(&runner, conf);
-    runner_add_args(&runner, "log-file",
-                    DEFAULT_LOG_FILE_DIRECTORY "/" GEOREP
-                                               "/${mastervol}/${eSlave}.log",
-                    ".", ".", NULL);
+    runner_add_arg(&runner, "log-file");
+    runner_argprintf(&runner, "%s/" GEOREP "/${mastervol}/${eSlave}.log",
+                     conf->logdir);
+    runner_add_args(&runner, ".", ".", NULL);
     RUN_GSYNCD_CMD;
 
     /* gluster-log-file */
     runinit_gsyncd_setrx(&runner, conf);
-    runner_add_args(&runner, "gluster-log-file",
-                    DEFAULT_LOG_FILE_DIRECTORY
-                    "/" GEOREP "/${mastervol}/${eSlave}${local_id}.gluster.log",
-                    ".", ".", NULL);
+    runner_add_arg(&runner, "gluster-log-file");
+    runner_argprintf(
+        &runner, "%s/" GEOREP "/${mastervol}/${eSlave}${local_id}.gluster.log",
+        conf->logdir);
+    runner_add_args(&runner, ".", ".", NULL);
     RUN_GSYNCD_CMD;
 
     /* ignore-deletes */
@@ -867,33 +886,36 @@ configure_syncdaemon(glusterd_conf_t *conf)
 
     /* log-file */
     runinit_gsyncd_setrx(&runner, conf);
-    runner_add_args(
-        &runner, "log-file",
-        DEFAULT_LOG_FILE_DIRECTORY
-        "/" GEOREP
+    runner_add_arg(&runner, "log-file");
+    runner_argprintf(
+        &runner,
+        "%s/" GEOREP
         "-slaves/${session_owner}:${local_node}${local_id}.${slavevol}.log",
-        ".", NULL);
+        conf->logdir);
+    runner_add_args(&runner, ".", ".", NULL);
     RUN_GSYNCD_CMD;
 
     /* MountBroker log-file */
     runinit_gsyncd_setrx(&runner, conf);
-    runner_add_args(
-        &runner, "log-file-mbr",
-        DEFAULT_LOG_FILE_DIRECTORY
-        "/" GEOREP
+    runner_add_arg(&runner, "log-file-mbr");
+    runner_argprintf(
+        &runner,
+        "%s/" GEOREP
         "-slaves/mbr/${session_owner}:${local_node}${local_id}.${slavevol}.log",
-        ".", NULL);
+        conf->logdir);
+    runner_add_args(&runner, ".", ".", NULL);
     RUN_GSYNCD_CMD;
 
     /* gluster-log-file */
     runinit_gsyncd_setrx(&runner, conf);
-    runner_add_args(
-        &runner, "gluster-log-file",
-        DEFAULT_LOG_FILE_DIRECTORY
-        "/" GEOREP
+    runner_add_arg(&runner, "gluster-log-file");
+    runner_argprintf(
+        &runner,
+        "%s/" GEOREP
         "-slaves/"
         "${session_owner}:${local_node}${local_id}.${slavevol}.gluster.log",
-        ".", NULL);
+        conf->logdir);
+    runner_add_args(&runner, ".", ".", NULL);
     RUN_GSYNCD_CMD;
 
 out:
@@ -1018,26 +1040,19 @@ _install_mount_spec(dict_t *opts, char *key, data_t *value, void *data)
     glusterd_conf_t *priv = THIS->private;
     char *label = NULL;
     gf_boolean_t georep = _gf_false;
-    gf_boolean_t ghadoop = _gf_false;
     char *pdesc = value->data;
     char *volname = NULL;
     int rv = 0;
     gf_mount_spec_t *mspec = NULL;
     char *user = NULL;
-    char *volfile_server = NULL;
 
     label = strtail(key, "mountbroker.");
 
-    /* check for presence of geo-rep/hadoop label */
+    /* check for presence of geo-rep label */
     if (!label) {
         label = strtail(key, "mountbroker-" GEOREP ".");
         if (label)
             georep = _gf_true;
-        else {
-            label = strtail(key, "mountbroker-" GHADOOP ".");
-            if (label)
-                ghadoop = _gf_true;
-        }
     }
 
     if (!label)
@@ -1048,7 +1063,7 @@ _install_mount_spec(dict_t *opts, char *key, data_t *value, void *data)
         goto err;
     mspec->label = label;
 
-    if (georep || ghadoop) {
+    if (georep) {
         volname = gf_strdup(pdesc);
         if (!volname)
             goto err;
@@ -1059,18 +1074,7 @@ _install_mount_spec(dict_t *opts, char *key, data_t *value, void *data)
         } else
             user = label;
 
-        if (georep)
-            rv = make_georep_mountspec(mspec, volname, user);
-
-        if (ghadoop) {
-            volfile_server = strchr(user, ':');
-            if (volfile_server)
-                *volfile_server++ = '\0';
-            else
-                volfile_server = "localhost";
-
-            rv = make_ghadoop_mountspec(mspec, volname, user, volfile_server);
-        }
+        rv = make_georep_mountspec(mspec, volname, user, priv->logdir);
 
         GF_FREE(volname);
         if (rv != 0)
@@ -1304,7 +1308,7 @@ glusterd_init_var_run_dirs(xlator_t *this, char *var_run_dir,
 
     if ((-1 == ret) && (ENOENT == errno)) {
         /* Create missing dirs */
-        ret = mkdir_p(abs_path, 0777, _gf_true);
+        ret = mkdir_p(abs_path, 0755, _gf_true);
 
         if (-1 == ret) {
             gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1391,6 +1395,9 @@ init(xlator_t *this)
     char rundir[PATH_MAX] = {
         0,
     };
+    char logdir[VALID_GLUSTERD_PATHMAX] = {
+        0,
+    };
     char cmd_log_filename[PATH_MAX] = {
         0,
     };
@@ -1439,6 +1446,26 @@ init(xlator_t *this)
     if (len < 0 || len >= PATH_MAX)
         exit(2);
 
+    dir_data = dict_get(this->options, "cluster-test-mode");
+    if (!dir_data) {
+        /* Use default working dir */
+        len = snprintf(logdir, VALID_GLUSTERD_PATHMAX, "%s",
+                       DEFAULT_LOG_FILE_DIRECTORY);
+    } else {
+        len = snprintf(logdir, VALID_GLUSTERD_PATHMAX, "%s", dir_data->data);
+        gf_msg(this->name, GF_LOG_INFO, 0, GD_MSG_CLUSTER_RC_ENABLE,
+               "cluster-test-mode is enabled logdir is %s", dir_data->data);
+    }
+    if (len < 0 || len >= PATH_MAX)
+        exit(2);
+
+    ret = mkdir_p(logdir, 0777, _gf_true);
+    if ((ret == -1) && (EEXIST != errno)) {
+        gf_msg(THIS->name, GF_LOG_ERROR, errno, GD_MSG_CREATE_DIR_FAILED,
+               "Unable to create log dir %s", logdir);
+        exit(1);
+    }
+
     dir_data = dict_get(this->options, "working-directory");
 
     if (!dir_data) {
@@ -1466,7 +1493,7 @@ init(xlator_t *this)
     }
 
     if ((-1 == ret) && (ENOENT == errno)) {
-        ret = mkdir_p(workdir, 0777, _gf_true);
+        ret = mkdir_p(workdir, 0755, _gf_true);
 
         if (-1 == ret) {
             gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1513,7 +1540,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    ret = mkdir_p(GLUSTER_SHARED_STORAGE_BRICK_DIR, 0777, _gf_true);
+    ret = mkdir_p(GLUSTER_SHARED_STORAGE_BRICK_DIR, 0755, _gf_true);
     if (ret) {
         gf_msg(this->name, GF_LOG_CRITICAL, 0, GD_MSG_DIR_OP_FAILED,
                "Unable to create "
@@ -1537,6 +1564,7 @@ init(xlator_t *this)
         exit(1);
     }
 
+#ifdef BUILD_GNFS
     ret = glusterd_init_var_run_dirs(this, rundir, GLUSTERD_NFS_RUN_DIR);
     if (ret) {
         gf_msg(this->name, GF_LOG_CRITICAL, 0, GD_MSG_CREATE_DIR_FAILED,
@@ -1544,6 +1572,7 @@ init(xlator_t *this)
                "nfs running directory");
         exit(1);
     }
+#endif
 
     ret = glusterd_init_var_run_dirs(this, rundir, GLUSTERD_QUOTAD_RUN_DIR);
     if (ret) {
@@ -1553,8 +1582,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    snprintf(cmd_log_filename, PATH_MAX, "%s/cmd_history.log",
-             DEFAULT_LOG_FILE_DIRECTORY);
+    snprintf(cmd_log_filename, PATH_MAX, "%s/cmd_history.log", logdir);
     ret = gf_cmd_log_init(cmd_log_filename);
 
     if (ret == -1) {
@@ -1568,7 +1596,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
 
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1584,7 +1612,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
 
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1599,7 +1627,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
 
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1614,7 +1642,7 @@ init(xlator_t *this)
         exit(1);
     }
 
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
 
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
@@ -1624,13 +1652,12 @@ init(xlator_t *this)
         exit(1);
     }
 
-    len = snprintf(storedir, sizeof(storedir), "%s/bricks",
-                   DEFAULT_LOG_FILE_DIRECTORY);
+    len = snprintf(storedir, sizeof(storedir), "%s/bricks", logdir);
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
 
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create logs directory %s"
@@ -1639,11 +1666,12 @@ init(xlator_t *this)
         exit(1);
     }
 
+#ifdef BUILD_GNFS
     len = snprintf(storedir, sizeof(storedir), "%s/nfs", workdir);
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create nfs directory %s"
@@ -1651,12 +1679,12 @@ init(xlator_t *this)
                storedir, errno);
         exit(1);
     }
-
+#endif
     len = snprintf(storedir, sizeof(storedir), "%s/bitd", workdir);
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create bitrot directory %s", storedir);
@@ -1667,7 +1695,7 @@ init(xlator_t *this)
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create scrub directory %s", storedir);
@@ -1678,7 +1706,7 @@ init(xlator_t *this)
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create glustershd directory %s"
@@ -1691,7 +1719,7 @@ init(xlator_t *this)
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create quotad directory %s"
@@ -1704,7 +1732,7 @@ init(xlator_t *this)
     if ((len < 0) || (len >= sizeof(storedir))) {
         exit(1);
     }
-    ret = sys_mkdir(storedir, 0777);
+    ret = sys_mkdir(storedir, 0755);
     if ((-1 == ret) && (errno != EEXIST)) {
         gf_msg(this->name, GF_LOG_CRITICAL, errno, GD_MSG_CREATE_DIR_FAILED,
                "Unable to create glustershd directory %s"
@@ -1840,7 +1868,13 @@ init(xlator_t *this)
     /* coverity[BUFFER_SIZE_WARNING] */
     (void)strncpy(conf->rundir, rundir, sizeof(conf->rundir));
 
+    /* coverity[BUFFER_SIZE_WARNING] */
+    (void)strncpy(conf->logdir, logdir, sizeof(conf->logdir));
+
     synclock_init(&conf->big_lock, SYNC_LOCK_RECURSIVE);
+    synccond_init(&conf->cond_restart_bricks);
+    synccond_init(&conf->cond_restart_shd);
+    synccond_init(&conf->cond_blockers);
     pthread_mutex_init(&conf->xprt_lock, NULL);
     INIT_LIST_HEAD(&conf->xprt_list);
     pthread_mutex_init(&conf->import_volumes, NULL);
@@ -1895,7 +1929,9 @@ init(xlator_t *this)
     glusterd_mgmt_v3_lock_timer_init();
     glusterd_txn_opinfo_dict_init();
 
+#ifdef BUILD_GNFS
     glusterd_nfssvc_build(&conf->nfs_svc);
+#endif
     glusterd_quotadsvc_build(&conf->quotad_svc);
     glusterd_bitdsvc_build(&conf->bitd_svc);
     glusterd_scrubsvc_build(&conf->scrub_svc);
@@ -2156,10 +2192,6 @@ struct volume_options options[] = {
         .key = {"mountbroker-" GEOREP ".*"},
         .type = GF_OPTION_TYPE_ANY,
     },
-    {
-        .key = {"mountbroker-" GHADOOP ".*"},
-        .type = GF_OPTION_TYPE_ANY,
-    },
     {
         .key = {GEOREP "-log-group"},
         .type = GF_OPTION_TYPE_ANY,
