@@ -78,6 +78,20 @@ typedef struct fuse_in_header fuse_in_header_t;
 typedef void(fuse_handler_t)(xlator_t *this, fuse_in_header_t *finh, void *msg,
                              struct iobuf *iobuf);
 
+enum fusedev_errno {
+    FUSEDEV_ENOENT,
+    FUSEDEV_ENOTDIR,
+    FUSEDEV_ENODEV,
+    FUSEDEV_EPERM,
+    FUSEDEV_ENOMEM,
+    FUSEDEV_ENOTCONN,
+    FUSEDEV_ECONNREFUSED,
+    FUSEDEV_EOVERFLOW,
+    FUSEDEV_EBUSY,
+    FUSEDEV_ENOTEMPTY,
+    FUSEDEV_EMAXPLUS
+};
+
 struct fuse_private {
     int fd;
     uint32_t proto_minor;
@@ -139,7 +153,7 @@ struct fuse_private {
     pthread_cond_t invalidate_cond;
     pthread_mutex_t invalidate_mutex;
     gf_boolean_t reverse_fuse_thread_started;
-
+    uint64_t invalidate_count;
     /* For communicating with separate mount thread. */
     int status_pipe[2];
 
@@ -189,19 +203,31 @@ struct fuse_private {
     gf_boolean_t flush_handle_interrupt;
     gf_boolean_t fuse_auto_inval;
 
-    /* LRU Limit, if not set, default is 128k for now */
+    /* LRU Limit, if not set, default is 64k for now */
     uint32_t lru_limit;
+    uint32_t invalidate_limit;
+    uint32_t fuse_dev_eperm_ratelimit_ns;
+
+    /* counters for fusdev errnos */
+    uint8_t fusedev_errno_cnt[FUSEDEV_EMAXPLUS];
+    pthread_mutex_t fusedev_errno_cnt_mutex;
 };
 typedef struct fuse_private fuse_private_t;
 
+typedef uint64_t errnomask_t[2];
+#define MASK_ERRNO(mask, n) ((mask)[(n) >> 6] |= ((uint64_t)1 << ((n)&63)))
+#define GET_ERRNO_MASK(mask, n) ((mask)[(n) >> 6] & ((uint64_t)1 << ((n)&63)))
+#define ERRNOMASK_MAX (64 * (sizeof(errnomask_t) / sizeof(uint64_t)))
+
 #define INVAL_BUF_SIZE                                                         \
     (sizeof(struct fuse_out_header) +                                          \
      max(sizeof(struct fuse_notify_inval_inode_out),                           \
          sizeof(struct fuse_notify_inval_entry_out) + NAME_MAX + 1))
 
 struct fuse_invalidate_node {
-    char inval_buf[INVAL_BUF_SIZE];
+    errnomask_t errnomask;
     struct list_head next;
+    char inval_buf[INVAL_BUF_SIZE];
 };
 typedef struct fuse_invalidate_node fuse_invalidate_node_t;
 
@@ -209,6 +235,7 @@ struct fuse_timed_message {
     struct fuse_out_header fuse_out_header;
     void *fuse_message_body;
     struct timespec scheduled_ts;
+    errnomask_t errnomask;
     struct list_head next;
 };
 typedef struct fuse_timed_message fuse_timed_message_t;
