@@ -12,25 +12,26 @@
  * Every other protocol version gets initialized from here.
  */
 
-#include "defaults.h"
+#include <glusterfs/defaults.h>
 #include "rpcsvc.h"
-#include "dict.h"
-#include "xlator.h"
+#include <glusterfs/dict.h>
+#include <glusterfs/xlator.h>
 #include "nfs.h"
-#include "mem-pool.h"
-#include "logging.h"
+#include <glusterfs/mem-pool.h>
+#include <glusterfs/logging.h>
 #include "nfs-fops.h"
 #include "mount3.h"
 #include "nfs3.h"
 #include "nfs-mem-types.h"
 #include "nfs3-helpers.h"
 #include "nlm4.h"
-#include "options.h"
+#include <glusterfs/options.h>
 #include "acl3.h"
 #include "rpc-drc.h"
-#include "syscall.h"
+#include <glusterfs/syscall.h>
 #include "rpcsvc.h"
 #include "nfs-messages.h"
+#include "glusterfs/statedump.h"
 
 #define OPT_SERVER_AUX_GIDS "nfs.server-aux-gids"
 #define OPT_SERVER_GID_CACHE_TIMEOUT "nfs.server.aux-gid-timeout"
@@ -1114,7 +1115,7 @@ nfs_init_state(xlator_t *this)
 
     GF_OPTION_INIT("nfs.event-threads", nfs->event_threads, uint32,
                    free_foppool);
-    event_reconfigure_threads(this->ctx->event_pool, nfs->event_threads);
+    gf_event_reconfigure_threads(this->ctx->event_pool, nfs->event_threads);
 
     this->private = (void *)nfs;
     INIT_LIST_HEAD(&nfs->versions);
@@ -1337,7 +1338,7 @@ nfs_reconfigure_state(xlator_t *this, dict_t *options)
 
     GF_OPTION_RECONF("nfs.event-threads", nfs->event_threads, options, uint32,
                      out);
-    event_reconfigure_threads(this->ctx->event_pool, nfs->event_threads);
+    gf_event_reconfigure_threads(this->ctx->event_pool, nfs->event_threads);
 
     ret = 0;
 out:
@@ -1529,7 +1530,7 @@ notify(xlator_t *this, int32_t event, void *data, ...)
     return 0;
 }
 
-int
+void
 fini(xlator_t *this)
 {
     struct nfs_state *nfs = NULL;
@@ -1539,7 +1540,7 @@ fini(xlator_t *this)
     gf_msg_debug(GF_NFS, 0, "NFS service going down");
     nfs_deinit_versions(&nfs->versions, this);
     GF_FREE(this->instance_name);
-    return 0;
+    return;
 }
 
 int32_t
@@ -1551,7 +1552,7 @@ nfs_forget(xlator_t *this, inode_t *inode)
     if (inode_ctx_del(inode, this, &ctx))
         return -1;
 
-    ictx = (struct nfs_inode_ctx *)ctx;
+    ictx = (struct nfs_inode_ctx *)(uintptr_t)ctx;
     GF_FREE(ictx);
 
     return 0;
@@ -1667,6 +1668,20 @@ out:
     return ret;
 }
 
+int32_t
+nfs_itable_dump(xlator_t *this)
+{
+    if (!this)
+        return -1;
+
+    if (this->next && this->next->itable) {
+        gf_proc_dump_add_section("xlator.nfs.itable");
+        inode_table_dump(this->next->itable, "xlator.nfs.itable");
+    }
+
+    return 0;
+}
+
 struct xlator_cbks cbks = {
     .forget = nfs_forget,
 };
@@ -1676,6 +1691,7 @@ struct xlator_fops fops;
 struct xlator_dumpops dumpops = {
     .priv = nfs_priv,
     .priv_to_dict = nfs_priv_to_dict,
+    .inode = nfs_itable_dump,
 };
 
 /* TODO: If needed, per-volume options below can be extended to be export
@@ -2030,7 +2046,7 @@ struct volume_options options[] = {
         .type = GF_OPTION_TYPE_SIZET,
         .min = 1,
         .max = 32,
-        .default_value = "1",
+        .default_value = "2",
         .description = "Specifies the number of event threads to execute in"
                        "in parallel. Larger values would help process"
                        " responses faster, depending on available processing"
@@ -2040,3 +2056,18 @@ struct volume_options options[] = {
     },
     {.key = {NULL}},
 };
+
+xlator_api_t xlator_api = {
+    .init = init,
+    .fini = fini,
+    .notify = notify,
+    .reconfigure = reconfigure,
+    .mem_acct_init = mem_acct_init,
+    .op_version = {1},
+    .dumpops = &dumpops,
+    .fops = &fops,
+    .cbks = &cbks,
+    .options = options,
+    .identifier = "gnfs",
+    .category = GF_MAINTAINED,
+};
