@@ -39,22 +39,18 @@ typedef enum {
 #define CLIENT_POST_FOP(fop, this_rsp_u, this_args_cbk, params...)             \
     do {                                                                       \
         gf_common_rsp *_this_rsp = &CPD_RSP_FIELD(this_rsp_u, fop);            \
-        int _op_ret = 0;                                                       \
-        int _op_errno = 0;                                                     \
                                                                                \
-        _op_ret = _this_rsp->op_ret;                                           \
-        _op_errno = gf_error_to_errno(_this_rsp->op_errno);                    \
+        int _op_ret = _this_rsp->op_ret;                                       \
+        int _op_errno = gf_error_to_errno(_this_rsp->op_errno);                \
         args_##fop##_cbk_store(this_args_cbk, _op_ret, _op_errno, params);     \
     } while (0)
 
 #define CLIENT_POST_FOP_TYPE(fop, this_rsp_u, this_args_cbk, params...)        \
     do {                                                                       \
         gfs3_##fop##_rsp *_this_rsp = &CPD_RSP_FIELD(this_rsp_u, fop);         \
-        int _op_ret = 0;                                                       \
-        int _op_errno = 0;                                                     \
                                                                                \
-        _op_ret = _this_rsp->op_ret;                                           \
-        _op_errno = gf_error_to_errno(_this_rsp->op_errno);                    \
+        int _op_ret = _this_rsp->op_ret;                                       \
+        int _op_errno = gf_error_to_errno(_this_rsp->op_errno);                \
         args_##fop##_cbk_store(this_args_cbk, _op_ret, _op_errno, params);     \
     } while (0)
 
@@ -67,10 +63,8 @@ typedef enum {
             goto label;                                                        \
         }                                                                      \
         if (remote_fd == -1) {                                                 \
-            gf_msg(xl->name, GF_LOG_WARNING, EBADFD, PC_MSG_BAD_FD,            \
-                   " (%s) "                                                    \
-                   "remote_fd is -1. EBADFD",                                  \
-                   uuid_utoa(fd->inode->gfid));                                \
+            gf_smsg(xl->name, GF_LOG_WARNING, EBADFD, PC_MSG_BAD_FD,           \
+                    "gfid=%s", uuid_utoa(fd->inode->gfid), NULL);              \
             op_errno = EBADFD;                                                 \
             goto label;                                                        \
         }                                                                      \
@@ -155,6 +149,16 @@ typedef struct clnt_conf {
                                           compltely, ie client_fini_complete
                                           to return*/
     gf_boolean_t fini_completed;
+    gf_boolean_t strict_locks; /* When set, doesn't reopen saved fds after
+                                  reconnect if POSIX locks are held on them.
+                                  Hence subsequent operations on these fds will
+                                  fail. This is necessary for stricter lock
+                                  complaince as bricks cleanup any granted
+                                  locks when a client disconnects.
+                               */
+
+    gf_boolean_t connection_to_brick; /*True from attempt to connect to brick
+                                        till disconnection to brick*/
 } clnt_conf_t;
 
 typedef struct _client_fd_ctx {
@@ -310,8 +314,6 @@ void
 client_save_number_fds(clnt_conf_t *conf, int count);
 int
 dump_client_locks(inode_t *inode);
-int
-client_notify_parents_child_up(xlator_t *this);
 int32_t
 is_client_dump_locks_cmd(char *name);
 int32_t
@@ -319,12 +321,6 @@ client_dump_locks(char *name, inode_t *inode, dict_t *dict);
 int
 client_fdctx_destroy(xlator_t *this, clnt_fd_ctx_t *fdctx);
 
-int32_t
-client_type_to_gf_type(short l_type);
-
-int
-client_mark_fd_bad(xlator_t *this);
-
 int
 client_fd_lk_list_empty(fd_lk_ctx_t *lk_ctx, gf_boolean_t use_try_lock);
 void
@@ -388,4 +384,11 @@ clnt_readdir_rsp_cleanup_v2(gfx_readdir_rsp *rsp);
 int
 clnt_readdirp_rsp_cleanup_v2(gfx_readdirp_rsp *rsp);
 
+int
+client_add_lock_for_recovery(fd_t *fd, struct gf_flock *flock,
+                             gf_lkowner_t *owner, int32_t cmd);
+
+int
+client_is_setlk(int32_t cmd);
+
 #endif /* !_CLIENT_H */
