@@ -48,7 +48,6 @@
 #include <glusterfs/glusterfs-acl.h>
 #include "posix-gfid-path.h"
 #include <glusterfs/events.h>
-#include "glusterfsd.h"
 #include "glusterfs/syncop.h"
 #include "timer-wheel.h"
 #include <sys/types.h>
@@ -56,20 +55,20 @@
 char *marker_xattrs[] = {"trusted.glusterfs.quota.*",
                          "trusted.glusterfs.*.xtime", NULL};
 
-char *marker_contri_key = "trusted.*.*.contri";
+static char *marker_contri_key = "trusted.*.*.contri";
 
 static char *posix_ignore_xattrs[] = {"gfid-req",
+                                      GLUSTERFS_INTERNAL_FOP_KEY,
                                       GLUSTERFS_ENTRYLK_COUNT,
                                       GLUSTERFS_INODELK_COUNT,
                                       GLUSTERFS_POSIXLK_COUNT,
                                       GLUSTERFS_PARENT_ENTRYLK,
                                       GF_GFIDLESS_LOOKUP,
                                       GLUSTERFS_INODELK_DOM_COUNT,
-                                      GLUSTERFS_INTERNAL_FOP_KEY,
                                       NULL};
 
-static char *list_xattr_ignore_xattrs[] = {
-    GF_SELINUX_XATTR_KEY, GF_XATTR_VOL_ID_KEY, GFID_XATTR_KEY, NULL};
+static char *list_xattr_ignore_xattrs[] = {GFID_XATTR_KEY, GF_XATTR_VOL_ID_KEY,
+                                           GF_SELINUX_XATTR_KEY, NULL};
 
 gf_boolean_t
 posix_special_xattr(char **pattern, char *key)
@@ -138,9 +137,6 @@ posix_handle_georep_xattrs(call_frame_t *frame, const char *name, int *op_errno,
     static const char *georep_xattr[] = {
         "*.glusterfs.*.stime", "*.glusterfs.*.xtime",
         "*.glusterfs.*.entry_stime", "*.glusterfs.volume-mark.*", NULL};
-    if (frame && frame->root) {
-        pid = frame->root->pid;
-    }
 
     if (!name) {
         /* No need to do anything here */
@@ -148,6 +144,10 @@ posix_handle_georep_xattrs(call_frame_t *frame, const char *name, int *op_errno,
         goto out;
     }
 
+    if (frame && frame->root) {
+        pid = frame->root->pid;
+    }
+
     if (pid == GF_CLIENT_PID_GSYNCD && is_getxattr) {
         filter_xattr = _gf_false;
 
@@ -213,14 +213,11 @@ posix_xattr_ignorable(char *key)
 static int
 _posix_xattr_get_set_from_backend(posix_xattr_filler_t *filler, char *key)
 {
-    ssize_t xattr_size = -1;
-    int ret = 0;
+    ssize_t xattr_size = 256; /* guesstimated initial size of xattr */
+    int ret = -1;
     char *value = NULL;
-    char val_buf[256] = {0};
-    gf_boolean_t have_val = _gf_false;
 
     if (!gf_is_valid_xattr_namespace(key)) {
-        ret = -1;
         goto out;
     }
 
@@ -229,46 +226,54 @@ _posix_xattr_get_set_from_backend(posix_xattr_filler_t *filler, char *key)
      * of getxattr with NULL buf to find the length and then getxattr with
      * allocated buf to fill the data. This way we reduce lot of getxattrs.
      */
-    if (filler->real_path)
-        xattr_size = sys_lgetxattr(filler->real_path, key, val_buf,
-                                   sizeof(val_buf) - 1);
-    else
-        xattr_size = sys_fgetxattr(filler->fdnum, key, val_buf,
-                                   sizeof(val_buf) - 1);
 
-    if (xattr_size >= 0) {
-        have_val = _gf_true;
-    } else if (xattr_size == -1 && errno != ERANGE) {
-        ret = -1;
+    value = GF_MALLOC(xattr_size + 1, gf_posix_mt_char);
+    if (!value) {
         goto out;
     }
 
-    if (have_val) {
-        /*No need to do getxattr*/
-    } else if (filler->real_path) {
-        xattr_size = sys_lgetxattr(filler->real_path, key, NULL, 0);
-    } else {
-        xattr_size = sys_fgetxattr(filler->fdnum, key, NULL, 0);
-    }
+    if (filler->real_path)
+        xattr_size = sys_lgetxattr(filler->real_path, key, value, xattr_size);
+    else
+        xattr_size = sys_fgetxattr(filler->fdnum, key, value, xattr_size);
 
-    if (xattr_size != -1) {
-        value = GF_MALLOC(xattr_size + 1, gf_posix_mt_char);
-        if (!value)
+    if (xattr_size == -1) {
+        if (value) {
+            GF_FREE(value);
+            value = NULL;
+        }
+        /* xattr_size == -1 - failed to fetch the xattr with
+         * current settings.
+         * If it was not because value was too small, abort
+         */
+        if (errno != ERANGE) {
             goto out;
+        }
 
-        if (have_val) {
-            memcpy(value, val_buf, xattr_size);
+        /* Get the real length needed */
+        if (filler->real_path) {
+            xattr_size = sys_lgetxattr(filler->real_path, key, NULL, 0);
         } else {
-            bzero(value, xattr_size + 1);
-            if (filler->real_path) {
-                xattr_size = sys_lgetxattr(filler->real_path, key, value,
-                                           xattr_size);
-            } else {
-                xattr_size = sys_fgetxattr(filler->fdnum, key, value,
-                                           xattr_size);
-            }
+            xattr_size = sys_fgetxattr(filler->fdnum, key, NULL, 0);
         }
         if (xattr_size == -1) {
+            goto out;
+        }
+
+        value = GF_MALLOC(xattr_size + 1, gf_posix_mt_char);
+        if (!value) {
+            goto out;
+        }
+
+        if (filler->real_path) {
+            xattr_size = sys_lgetxattr(filler->real_path, key, value,
+                                       xattr_size);
+        } else {
+            xattr_size = sys_fgetxattr(filler->fdnum, key, value, xattr_size);
+        }
+        if (xattr_size == -1) {
+            GF_FREE(value);
+            value = NULL;
             if (filler->real_path)
                 gf_msg(filler->this->name, GF_LOG_WARNING, 0,
                        P_MSG_XATTR_FAILED, "getxattr failed. path: %s, key: %s",
@@ -277,24 +282,25 @@ _posix_xattr_get_set_from_backend(posix_xattr_filler_t *filler, char *key)
                 gf_msg(filler->this->name, GF_LOG_WARNING, 0,
                        P_MSG_XATTR_FAILED, "getxattr failed. gfid: %s, key: %s",
                        uuid_utoa(filler->fd->inode->gfid), key);
-            GF_FREE(value);
             goto out;
         }
+    }
 
-        value[xattr_size] = '\0';
-        ret = dict_set_bin(filler->xattr, key, value, xattr_size);
-        if (ret < 0) {
-            if (filler->real_path)
-                gf_msg_debug(filler->this->name, 0,
-                             "dict set failed. path: %s, key: %s",
-                             filler->real_path, key);
-            else
-                gf_msg_debug(filler->this->name, 0,
-                             "dict set failed. gfid: %s, key: %s",
-                             uuid_utoa(filler->fd->inode->gfid), key);
+    value[xattr_size] = '\0';
+    ret = dict_set_bin(filler->xattr, key, value, xattr_size);
+
+    if (ret < 0) {
+        if (value)
             GF_FREE(value);
-            goto out;
-        }
+        if (filler->real_path)
+            gf_msg_debug(filler->this->name, 0,
+                         "dict set failed. path: %s, key: %s",
+                         filler->real_path, key);
+        else
+            gf_msg_debug(filler->this->name, 0,
+                         "dict set failed. gfid: %s, key: %s",
+                         uuid_utoa(filler->fd->inode->gfid), key);
+        goto out;
     }
     ret = 0;
 out:
@@ -359,11 +365,10 @@ _posix_get_marker_all_contributions(posix_xattr_filler_t *filler)
     list_offset = 0;
 
     while (remaining_size > 0) {
-        snprintf(key, sizeof(key), "%s", list + list_offset);
+        len = snprintf(key, sizeof(key), "%s", list + list_offset);
         if (fnmatch(marker_contri_key, key, 0) == 0) {
-            ret = _posix_xattr_get_set_from_backend(filler, key);
+            (void)_posix_xattr_get_set_from_backend(filler, key);
         }
-        len = strlen(key);
         remaining_size -= (len + 1);
         list_offset += (len + 1);
     }
@@ -418,7 +423,7 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
 {
     posix_xattr_filler_t *filler = xattrargs;
     int ret = -1;
-    int len;
+    int len = 0;
     char *databuf = NULL;
     int _fd = -1;
     ssize_t req_size = 0;
@@ -433,8 +438,11 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
 
     if (posix_xattr_ignorable(key))
         goto out;
+
+    len = strlen(key);
     /* should size be put into the data_t ? */
-    if (!strcmp(key, GF_CONTENT_KEY) && IA_ISREG(filler->stbuf->ia_type)) {
+    if ((filler->stbuf != NULL && IA_ISREG(filler->stbuf->ia_type)) &&
+        (len == SLEN(GF_CONTENT_KEY) && !strcmp(key, GF_CONTENT_KEY))) {
         if (!filler->real_path)
             goto out;
 
@@ -499,7 +507,8 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
                 sys_close(_fd);
             GF_FREE(databuf);
         }
-    } else if (!strcmp(key, GLUSTERFS_OPEN_FD_COUNT)) {
+    } else if (len == SLEN(GLUSTERFS_OPEN_FD_COUNT) &&
+               !strcmp(key, GLUSTERFS_OPEN_FD_COUNT)) {
         inode = _get_filler_inode(filler);
         if (!inode || gf_uuid_is_null(inode->gfid))
             goto out;
@@ -508,7 +517,8 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
             gf_msg(filler->this->name, GF_LOG_WARNING, 0, P_MSG_DICT_SET_FAILED,
                    "Failed to set dictionary value for %s", key);
         }
-    } else if (!strcmp(key, GLUSTERFS_ACTIVE_FD_COUNT)) {
+    } else if (len == SLEN(GLUSTERFS_ACTIVE_FD_COUNT) &&
+               !strcmp(key, GLUSTERFS_ACTIVE_FD_COUNT)) {
         inode = _get_filler_inode(filler);
         if (!inode || gf_uuid_is_null(inode->gfid))
             goto out;
@@ -517,7 +527,8 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
             gf_msg(filler->this->name, GF_LOG_WARNING, 0, P_MSG_DICT_SET_FAILED,
                    "Failed to set dictionary value for %s", key);
         }
-    } else if (!strcmp(key, GET_ANCESTRY_PATH_KEY)) {
+    } else if (len == SLEN(GET_ANCESTRY_PATH_KEY) &&
+               !strcmp(key, GET_ANCESTRY_PATH_KEY)) {
         /* As of now, the only consumers of POSIX_ANCESTRY_PATH attempt
          * fetching it via path-based fops. Hence, leaving it as it is
          * for now.
@@ -532,7 +543,7 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
             goto out;
         }
 
-        ret = dict_set_dynstr(filler->xattr, GET_ANCESTRY_PATH_KEY, path);
+        ret = dict_set_dynstr_sizen(filler->xattr, GET_ANCESTRY_PATH_KEY, path);
         if (ret < 0) {
             GF_FREE(path);
             goto out;
@@ -540,9 +551,10 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
 
     } else if (fnmatch(marker_contri_key, key, 0) == 0) {
         ret = _posix_get_marker_quota_contributions(filler, key);
-    } else if (strcmp(key, GF_REQUEST_LINK_COUNT_XDATA) == 0) {
-        ret = dict_set(filler->xattr, GF_REQUEST_LINK_COUNT_XDATA, data);
-    } else if (strcmp(key, GF_GET_SIZE) == 0) {
+    } else if (len == SLEN(GF_REQUEST_LINK_COUNT_XDATA) &&
+               strcmp(key, GF_REQUEST_LINK_COUNT_XDATA) == 0) {
+        ret = dict_set_sizen(filler->xattr, GF_REQUEST_LINK_COUNT_XDATA, data);
+    } else if (len == SLEN(GF_GET_SIZE) && strcmp(key, GF_GET_SIZE) == 0) {
         if (filler->stbuf && IA_ISREG(filler->stbuf->ia_type)) {
             ret = dict_set_uint64(filler->xattr, GF_GET_SIZE,
                                   filler->stbuf->ia_size);
@@ -567,7 +579,7 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
 
         /* ACL_TYPE_DEFAULT is not supported for non-directory, skip */
         if (!IA_ISDIR(stbuf.ia_type) &&
-            !strncmp(key, GF_POSIX_ACL_DEFAULT, strlen(GF_POSIX_ACL_DEFAULT)))
+            !strncmp(key, GF_POSIX_ACL_DEFAULT, SLEN(GF_POSIX_ACL_DEFAULT)))
             goto out;
 
         ret = posix_pacl_get(filler->real_path, filler->fdnum, key, &value);
@@ -579,7 +591,7 @@ _posix_xattr_get_set(dict_t *xattr_req, char *key, data_t *data,
             goto out;
         }
 
-        ret = dict_set_dynstr(filler->xattr, (char *)key, value);
+        ret = dict_set_dynstrn(filler->xattr, (char *)key, len, value);
         if (ret < 0) {
             GF_FREE(value);
             gf_msg(filler->this->name, GF_LOG_ERROR, errno,
@@ -812,6 +824,11 @@ posix_pstat(xlator_t *this, inode_t *inode, uuid_t gfid, const char *path,
             gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_LSTAT_FAILED,
                    "lstat failed on %s", path);
             errno = op_errno; /*gf_msg could have changed errno*/
+        } else {
+            op_errno = errno;
+            gf_msg_debug(this->name, 0, "lstat failed on %s (%s)", path,
+                         strerror(errno));
+            errno = op_errno; /*gf_msg could have changed errno*/
         }
         goto out;
     }
@@ -889,18 +906,17 @@ out:
 }
 
 static void
-_handle_list_xattr(dict_t *xattr_req, const char *real_path, int fdnum,
-                   posix_xattr_filler_t *filler)
+_handle_list_xattr(posix_xattr_filler_t *filler)
 {
     int32_t list_offset = 0;
     ssize_t remaining_size = 0;
     char *key = NULL;
     int len;
 
-    list_offset = 0;
     remaining_size = filler->list_size;
     while (remaining_size > 0) {
         key = filler->list + list_offset;
+        len = strlen(key);
 
         if (gf_get_index_by_elem(list_xattr_ignore_xattrs, key) >= 0)
             goto next;
@@ -914,12 +930,11 @@ _handle_list_xattr(dict_t *xattr_req, const char *real_path, int fdnum,
         if (posix_is_gfid2path_xattr(key))
             goto next;
 
-        if (dict_get(filler->xattr, key))
+        if (dict_getn(filler->xattr, key, len))
             goto next;
 
         (void)_posix_xattr_get_set_from_backend(filler, key);
     next:
-        len = strlen(key);
         remaining_size -= (len + 1);
         list_offset += (len + 1);
 
@@ -937,8 +952,8 @@ posix_xattr_fill(xlator_t *this, const char *real_path, loc_t *loc, fd_t *fd,
     };
     gf_boolean_t list = _gf_false;
 
-    if (dict_get(xattr_req, "list-xattr")) {
-        dict_del(xattr_req, "list-xattr");
+    if (dict_get_sizen(xattr_req, "list-xattr")) {
+        dict_del_sizen(xattr_req, "list-xattr");
         list = _gf_true;
     }
 
@@ -958,7 +973,7 @@ posix_xattr_fill(xlator_t *this, const char *real_path, loc_t *loc, fd_t *fd,
     _get_list_xattr(&filler);
     dict_foreach(xattr_req, _posix_xattr_get_set, &filler);
     if (list)
-        _handle_list_xattr(xattr_req, real_path, fdnum, &filler);
+        _handle_list_xattr(&filler);
 
     GF_FREE(filler.list);
 out:
@@ -1055,13 +1070,13 @@ verify_handle:
         ret = posix_handle_soft(this, path, loc, uuid_curr, &stat);
 
 out:
-    if (!(*op_errno))
+    if (ret && !(*op_errno))
         *op_errno = errno;
     return ret;
 }
 
 #ifdef HAVE_SYS_ACL_H
-int
+static int
 posix_pacl_set(const char *path, int fdnum, const char *key, const char *acl_s)
 {
     int ret = -1;
@@ -1209,10 +1224,11 @@ posix_handle_pair(xlator_t *this, loc_t *loc, const char *real_path, char *key,
         if (stbuf && IS_DHT_LINKFILE_MODE(stbuf))
             goto out;
         ret = posix_pacl_set(real_path, -1, key, value->data);
-    } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR, strlen(key)) && stbuf &&
-               IS_DHT_LINKFILE_MODE(stbuf)) {
+    } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR,
+                        SLEN(POSIX_ACL_ACCESS_XATTR)) &&
+               stbuf && IS_DHT_LINKFILE_MODE(stbuf)) {
         goto out;
-    } else if (!strncmp(key, GF_INTERNAL_CTX_KEY, strlen(key))) {
+    } else if (!strncmp(key, GF_INTERNAL_CTX_KEY, SLEN(GF_INTERNAL_CTX_KEY))) {
         /* ignore this key value pair */
         ret = 0;
         goto out;
@@ -1279,8 +1295,9 @@ posix_fhandle_pair(call_frame_t *frame, xlator_t *this, int fd, char *key,
     } else if (posix_is_gfid2path_xattr(key)) {
         ret = -ENOTSUP;
         goto out;
-    } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR, strlen(key)) && stbuf &&
-               IS_DHT_LINKFILE_MODE(stbuf)) {
+    } else if (!strncmp(key, POSIX_ACL_ACCESS_XATTR,
+                        SLEN(POSIX_ACL_ACCESS_XATTR)) &&
+               stbuf && IS_DHT_LINKFILE_MODE(stbuf)) {
         goto out;
     }
 
@@ -1339,7 +1356,7 @@ del_stale_dir_handle(xlator_t *this, uuid_t gfid)
         0,
     };
 
-    MAKE_HANDLE_GFID_PATH(hpath, this, gfid, NULL);
+    MAKE_HANDLE_GFID_PATH(hpath, this, gfid);
 
     /* check that it is valid directory handle */
     size = sys_lstat(hpath, &stbuf);
@@ -1449,12 +1466,24 @@ posix_janitor_task_done(int ret, call_frame_t *frame, void *data)
     this = data;
     priv = this->private;
 
+    pthread_mutex_lock(&priv->janitor_mutex);
+    {
+        if (priv->janitor_task_stop) {
+            priv->janitor_task_stop = _gf_false;
+            pthread_cond_signal(&priv->janitor_cond);
+            pthread_mutex_unlock(&priv->janitor_mutex);
+            goto out;
+        }
+    }
+    pthread_mutex_unlock(&priv->janitor_mutex);
+
     LOCK(&priv->lock);
     {
         __posix_janitor_timer_start(this);
     }
     UNLOCK(&priv->lock);
 
+out:
     return 0;
 }
 
@@ -1473,6 +1502,9 @@ posix_janitor_task(void *data)
     old_this = THIS;
     THIS = this;
 
+    if (!priv)
+        goto out;
+
     time(&now);
     if ((now - priv->last_landfill_check) > priv->janitor_sleep_duration) {
         if (priv->disable_landfill_purge) {
@@ -1492,6 +1524,7 @@ posix_janitor_task(void *data)
 
     THIS = old_this;
 
+out:
     return 0;
 }
 
@@ -1528,7 +1561,7 @@ __posix_janitor_timer_start(xlator_t *this)
     timer->expires = priv->janitor_sleep_duration;
     timer->function = posix_janitor_task_initator;
     timer->data = this;
-    gf_tw_add_timer(this->ctx->tw->timer_wheel, timer);
+    gf_tw_add_timer(glusterfs_ctx_tw_get(this->ctx), timer);
 
     return;
 }
@@ -1559,104 +1592,6 @@ unlock:
     return;
 }
 
-static struct posix_fd *
-janitor_get_next_fd(glusterfs_ctx_t *ctx, int32_t janitor_sleep)
-{
-    struct posix_fd *pfd = NULL;
-
-    struct timespec timeout;
-
-    pthread_mutex_lock(&ctx->janitor_lock);
-    {
-        if (list_empty(&ctx->janitor_fds)) {
-            time(&timeout.tv_sec);
-            timeout.tv_sec += janitor_sleep;
-            timeout.tv_nsec = 0;
-
-            pthread_cond_timedwait(&ctx->janitor_cond, &ctx->janitor_lock,
-                                   &timeout);
-            goto unlock;
-        }
-
-        pfd = list_entry(ctx->janitor_fds.next, struct posix_fd, list);
-
-        list_del(ctx->janitor_fds.next);
-    }
-unlock:
-    pthread_mutex_unlock(&ctx->janitor_lock);
-
-    return pfd;
-}
-
-static void *
-posix_ctx_janitor_thread_proc(void *data)
-{
-    xlator_t *this = NULL;
-    struct posix_fd *pfd;
-    glusterfs_ctx_t *ctx = NULL;
-    struct posix_private *priv = NULL;
-    int32_t sleep_duration = 0;
-
-    this = data;
-    ctx = THIS->ctx;
-    THIS = this;
-
-    priv = this->private;
-    sleep_duration = priv->janitor_sleep_duration;
-    while (1) {
-        pfd = janitor_get_next_fd(ctx, sleep_duration);
-        if (pfd) {
-            if (pfd->dir == NULL) {
-                gf_msg_trace(this->name, 0, "janitor: closing file fd=%d",
-                             pfd->fd);
-                sys_close(pfd->fd);
-            } else {
-                gf_msg_debug(this->name, 0, "janitor: closing dir fd=%p",
-                             pfd->dir);
-                sys_closedir(pfd->dir);
-            }
-
-            GF_FREE(pfd);
-        }
-    }
-
-    return NULL;
-}
-
-int
-posix_spawn_ctx_janitor_thread(xlator_t *this)
-{
-    struct posix_private *priv = NULL;
-    int ret = 0;
-    glusterfs_ctx_t *ctx = NULL;
-
-    priv = this->private;
-    ctx = THIS->ctx;
-
-    LOCK(&priv->lock);
-    {
-        if (!ctx->janitor) {
-            pthread_mutex_init(&ctx->janitor_lock, NULL);
-            pthread_cond_init(&ctx->janitor_cond, NULL);
-            INIT_LIST_HEAD(&ctx->janitor_fds);
-
-            ret = gf_thread_create(&ctx->janitor, NULL,
-                                   posix_ctx_janitor_thread_proc, this,
-                                   "posixctxjan");
-
-            if (ret) {
-                gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_THREAD_FAILED,
-                       "spawning janitor "
-                       "thread failed");
-                goto unlock;
-            }
-        }
-    }
-unlock:
-    UNLOCK(&priv->lock);
-    return ret;
-}
-
 static int
 is_fresh_file(int64_t sec, int64_t ns)
 {
@@ -1764,7 +1699,7 @@ posix_gfid_heal(xlator_t *this, const char *path, loc_t *loc, dict_t *xattr_req)
         }
     }
 
-    posix_gfid_set(this, path, loc, xattr_req, GF_CLIENT_PID_MAX, &ret);
+    (void)posix_gfid_set(this, path, loc, xattr_req, GF_CLIENT_PID_MAX, &ret);
     return 0;
 }
 
@@ -1885,8 +1820,8 @@ __posix_fd_ctx_get(fd_t *fd, xlator_t *this, struct posix_fd **pfd_p,
     if (!fd_is_anonymous(fd)) {
         gf_msg(this->name, GF_LOG_ERROR, 0, P_MSG_READ_FAILED,
                "Failed to get fd context for a non-anonymous fd, "
-               "file: %s, gfid: %s",
-               real_path, uuid_utoa(fd->inode->gfid));
+               "gfid: %s",
+               uuid_utoa(fd->inode->gfid));
         op_errno = EINVAL;
         goto out;
     }
@@ -1984,12 +1919,11 @@ posix_fd_ctx_get(fd_t *fd, xlator_t *this, struct posix_fd **pfd, int *op_errno)
     return ret;
 }
 
-int
-posix_fs_health_check(xlator_t *this)
+static int
+posix_fs_health_check(xlator_t *this, char *file_path)
 {
     struct posix_private *priv = NULL;
     int ret = -1;
-    char *subvol_path = NULL;
     char timestamp[256] = {
         0,
     };
@@ -1999,25 +1933,15 @@ posix_fs_health_check(xlator_t *this)
         0,
     };
     char buff[256] = {0};
-    char file_path[PATH_MAX] = {0};
     char *op = NULL;
     int op_errno = 0;
-    int cnt = 0;
+    int cnt;
     int timeout = 0;
     struct aiocb aiocb;
 
-    GF_VALIDATE_OR_GOTO(this->name, this, out);
     priv = this->private;
-    GF_VALIDATE_OR_GOTO("posix-helpers", priv, out);
 
-    subvol_path = priv->base_path;
     timeout = priv->health_check_timeout;
-    snprintf(file_path, sizeof(file_path) - 1, "%s/%s/health_check",
-             subvol_path, GF_HIDDEN_PATH);
-
-    time_sec = time(NULL);
-    gf_time_fmt(timestamp, sizeof timestamp, time_sec, gf_timefmt_FT);
-    timelen = strlen(timestamp);
 
     fd = open(file_path, O_CREAT | O_WRONLY | O_TRUNC, 0644);
     if (fd == -1) {
@@ -2025,6 +1949,11 @@ posix_fs_health_check(xlator_t *this)
         op = "open_for_write";
         goto out;
     }
+
+    time_sec = time(NULL);
+    gf_time_fmt(timestamp, sizeof timestamp, time_sec, gf_timefmt_FT);
+    timelen = strlen(timestamp);
+
     memset(&aiocb, 0, sizeof(struct aiocb));
     aiocb.aio_fildes = fd;
     aiocb.aio_buf = timestamp;
@@ -2036,6 +1965,7 @@ posix_fs_health_check(xlator_t *this)
         goto out;
     }
 
+    cnt = 0;
     /* Wait until write completion */
     while ((aio_error(&aiocb) == EINPROGRESS) && (++cnt <= timeout))
         sleep(1);
@@ -2104,14 +2034,20 @@ out:
     if (fd != -1) {
         sys_close(fd);
     }
+
     if (ret && file_path[0]) {
         gf_msg(this->name, GF_LOG_WARNING, errno, P_MSG_HEALTHCHECK_FAILED,
                "%s() on %s returned ret is %d error is %s", op, file_path, ret,
                ret != -1 ? strerror(ret) : strerror(op_errno));
-        gf_event(EVENT_POSIX_HEALTH_CHECK_FAILED,
-                 "op=%s;path=%s;error=%s;brick=%s:%s timeout is %d", op,
-                 file_path, strerror(op_errno), priv->hostname, priv->base_path,
-                 timeout);
+
+        if ((op_errno == EAGAIN) || (ret == EAGAIN)) {
+            ret = 0;
+        } else {
+            gf_event(EVENT_POSIX_HEALTH_CHECK_FAILED,
+                     "op=%s;path=%s;error=%s;brick=%s:%s timeout is %d", op,
+                     file_path, strerror(op_errno), priv->hostname,
+                     priv->base_path, timeout);
+        }
     }
     return ret;
 }
@@ -2119,30 +2055,30 @@ out:
 static void *
 posix_health_check_thread_proc(void *data)
 {
-    xlator_t *this = NULL;
-    struct posix_private *priv = NULL;
-    uint32_t interval = 0;
+    xlator_t *this = data;
+    struct posix_private *priv = this->private;
+    uint32_t interval = priv->health_check_interval;
     int ret = -1;
     xlator_t *top = NULL;
     xlator_t *victim = NULL;
     xlator_list_t **trav_p = NULL;
     int count = 0;
     gf_boolean_t victim_found = _gf_false;
-    glusterfs_ctx_t *ctx = NULL;
-
-    this = data;
-    priv = this->private;
-    ctx = THIS->ctx;
+    glusterfs_ctx_t *ctx = THIS->ctx;
+    char file_path[PATH_MAX];
 
     /* prevent races when the interval is updated */
-    interval = priv->health_check_interval;
     if (interval == 0)
         goto out;
 
+    snprintf(file_path, sizeof(file_path) - 1, "%s/%s/health_check",
+             priv->base_path, GF_HIDDEN_PATH);
+
     gf_msg_debug(this->name, 0,
                  "health-check thread started, "
+                 "on path %s, "
                  "interval = %d seconds",
-                 interval);
+                 file_path, interval);
     while (1) {
         /* aborting sleep() is a request to exit this thread, sleep()
          * will normally not return when cancelled */
@@ -2153,7 +2089,7 @@ posix_health_check_thread_proc(void *data)
         pthread_setcancelstate(PTHREAD_CANCEL_DISABLE, NULL);
 
         /* Do the health-check.*/
-        ret = posix_fs_health_check(this);
+        ret = posix_fs_health_check(this, file_path);
         if (ret < 0 && priv->health_check_active)
             goto abort;
         if (!priv->health_check_active)
@@ -2280,26 +2216,18 @@ posix_disk_space_check(xlator_t *this)
     struct posix_private *priv = NULL;
     char *subvol_path = NULL;
     int op_ret = 0;
-    uint64_t size = 0;
-    int percent = 0;
+    double size = 0;
+    double percent = 0;
     struct statvfs buf = {0};
-    uint64_t totsz = 0;
-    uint64_t freesz = 0;
+    double totsz = 0;
+    double freesz = 0;
 
-    GF_VALIDATE_OR_GOTO(this->name, this, out);
+    GF_VALIDATE_OR_GOTO("posix-helpers", this, out);
     priv = this->private;
     GF_VALIDATE_OR_GOTO(this->name, priv, out);
 
     subvol_path = priv->base_path;
 
-    if (priv->disk_reserve_size) {
-        size = priv->disk_reserve_size;
-    } else {
-        percent = priv->disk_reserve_percent;
-        totsz = (buf.f_blocks * buf.f_bsize);
-        size = ((totsz * percent) / 100);
-    }
-
     op_ret = sys_statvfs(subvol_path, &buf);
 
     if (op_ret == -1) {
@@ -2307,8 +2235,16 @@ posix_disk_space_check(xlator_t *this)
                "statvfs failed on %s", subvol_path);
         goto out;
     }
-    freesz = (buf.f_bfree * buf.f_bsize);
 
+    if (priv->disk_unit == 'p') {
+        percent = priv->disk_reserve;
+        totsz = (buf.f_blocks * buf.f_bsize);
+        size = ((totsz * percent) / 100);
+    } else {
+        size = priv->disk_reserve;
+    }
+
+    freesz = (buf.f_bfree * buf.f_bsize);
     if (freesz <= size) {
         priv->disk_space_full = 1;
     } else {
@@ -2496,7 +2432,7 @@ posix_fsyncer(void *d)
 
         count = posix_fsyncer_pick(this, &list);
 
-        usleep(priv->batch_fsync_delay_usec);
+        gf_nanosleep(priv->batch_fsync_delay_usec * GF_US_IN_NS);
 
         gf_msg_debug(this->name, 0, "picked %d fsyncs", count);
 
@@ -2768,10 +2704,10 @@ posix_resolve_dirgfid_to_path(const uuid_t dirgfid, const char *brick_path,
 
         linkname[len] = '\0';
 
-        pgfidstr = strtok_r(linkname + strlen("../../00/00/"), "/", &saveptr);
+        pgfidstr = strtok_r(linkname + SLEN("../../00/00/"), "/", &saveptr);
         dir_name = strtok_r(NULL, "/", &saveptr);
 
-        if (strlen(pre_dir_name) != 0) { /* Remove '/' at the end */
+        if (pre_dir_name[0] != '\0') { /* Remove '/' at the end */
             len = snprintf(result, PATH_MAX, "%s/%s", dir_name, pre_dir_name);
         } else {
             len = snprintf(result, PATH_MAX, "%s", dir_name);
@@ -2900,7 +2836,7 @@ posix_inode_ctx_get_all(inode_t *inode, xlator_t *this, posix_inode_ctx_t **ctx)
 gf_boolean_t
 posix_is_bulk_removexattr(char *name, dict_t *xdata)
 {
-    if (name && (strlen(name) == 0) && xdata)
+    if (name && (name[0] == '\0') && xdata)
         return _gf_true;
     return _gf_false;
 }
@@ -2980,7 +2916,7 @@ posix_check_internal_writes(xlator_t *this, fd_t *fd, int sysfd, dict_t *xdata)
 
     LOCK(&fd->inode->lock);
     {
-        val = dict_get(xdata, GF_PROTECT_FROM_EXTERNAL_WRITES);
+        val = dict_get_sizen(xdata, GF_PROTECT_FROM_EXTERNAL_WRITES);
         if (val) {
             ret = sys_fsetxattr(sysfd, GF_PROTECT_FROM_EXTERNAL_WRITES,
                                 val->data, val->len, 0);
@@ -2993,7 +2929,7 @@ posix_check_internal_writes(xlator_t *this, fd_t *fd, int sysfd, dict_t *xdata)
             goto out;
         }
 
-        if (dict_get(xdata, GF_AVOID_OVERWRITE)) {
+        if (dict_get_sizen(xdata, GF_AVOID_OVERWRITE)) {
             xattrsize = sys_fgetxattr(sysfd, GF_PROTECT_FROM_EXTERNAL_WRITES,
                                       NULL, 0);
             if ((xattrsize == -1) &&
@@ -3368,14 +3304,20 @@ posix_cs_maintenance(xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
 {
     gf_cs_obj_state state = GF_CS_ERROR;
     int ret = 0;
+    gf_boolean_t is_cs_obj_status = _gf_false;
+    gf_boolean_t is_cs_obj_repair = _gf_false;
 
-    if (!(dict_get(xattr_req, GF_CS_OBJECT_STATUS) ||
-          dict_get(xattr_req, GF_CS_OBJECT_REPAIR)))
+    if (dict_get_sizen(xattr_req, GF_CS_OBJECT_STATUS))
+        is_cs_obj_status = _gf_true;
+    if (dict_get_sizen(xattr_req, GF_CS_OBJECT_REPAIR))
+        is_cs_obj_repair = _gf_true;
+
+    if (!(is_cs_obj_status || is_cs_obj_repair))
         return 0;
 
     if (fd) {
         LOCK(&fd->inode->lock);
-        if (dict_get(xattr_req, GF_CS_OBJECT_STATUS)) {
+        if (is_cs_obj_status) {
             state = posix_cs_check_status(this, NULL, pfd, buf);
             gf_msg_debug(this->name, 0, "state : %d", state);
             ret = posix_cs_set_state(this, xattr_rsp, state, NULL, pfd);
@@ -3395,7 +3337,7 @@ posix_cs_maintenance(xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
             }
         }
 
-        if (dict_get(xattr_req, GF_CS_OBJECT_REPAIR)) {
+        if (is_cs_obj_repair) {
             state = posix_cs_check_status(this, NULL, pfd, buf);
             gf_msg_debug(this->name, 0, "state : %d", state);
 
@@ -3426,7 +3368,7 @@ posix_cs_maintenance(xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
         }
 
         LOCK(&loc->inode->lock);
-        if (dict_get(xattr_req, GF_CS_OBJECT_STATUS)) {
+        if (is_cs_obj_status) {
             state = posix_cs_check_status(this, realpath, NULL, buf);
             gf_msg_debug(this->name, 0, "state : %d", state);
             ret = posix_cs_set_state(this, xattr_rsp, state, realpath, NULL);
@@ -3446,7 +3388,7 @@ posix_cs_maintenance(xlator_t *this, fd_t *fd, loc_t *loc, int *pfd,
             }
         }
 
-        if (dict_get(xattr_req, GF_CS_OBJECT_REPAIR)) {
+        if (is_cs_obj_repair) {
             state = posix_cs_check_status(this, realpath, NULL, buf);
             gf_msg_debug(this->name, 0, "state : %d", state);
 
@@ -3512,8 +3454,7 @@ posix_update_iatt_buf(struct iatt *buf, int fd, char *loc, dict_t *xattr_req)
     if (!xattr_req)
         return;
 
-    if (!(dict_getn(xattr_req, GF_CS_OBJECT_STATUS,
-                    strlen(GF_CS_OBJECT_STATUS))))
+    if (!dict_get_sizen(xattr_req, GF_CS_OBJECT_STATUS))
         return;
 
     if (fd != -1) {
@@ -3550,3 +3491,80 @@ posix_update_iatt_buf(struct iatt *buf, int fd, char *loc, dict_t *xattr_req)
         }
     }
 }
+
+gf_boolean_t
+posix_is_layout_stale(dict_t *xdata, char *par_path, xlator_t *this)
+{
+    int op_ret = 0;
+    ssize_t size = 0;
+    char value_buf[4096] = {
+        0,
+    };
+    gf_boolean_t have_val = _gf_false;
+    data_t *arg_data = NULL;
+    char *xattr_name = NULL;
+    gf_boolean_t is_stale = _gf_false;
+
+    op_ret = dict_get_str_sizen(xdata, GF_PREOP_PARENT_KEY, &xattr_name);
+    if (xattr_name == NULL) {
+        op_ret = 0;
+        return is_stale;
+    }
+
+    arg_data = dict_get(xdata, xattr_name);
+    if (!arg_data) {
+        op_ret = 0;
+        dict_del_sizen(xdata, GF_PREOP_PARENT_KEY);
+        return is_stale;
+    }
+
+    size = sys_lgetxattr(par_path, xattr_name, value_buf,
+                         sizeof(value_buf) - 1);
+
+    if (size >= 0) {
+        have_val = _gf_true;
+    } else {
+        if (errno == ERANGE) {
+            gf_msg(this->name, GF_LOG_INFO, errno, P_MSG_PREOP_CHECK_FAILED,
+                   "getxattr on key (%s) path (%s) failed due to"
+                   " buffer overflow",
+                   xattr_name, par_path);
+            size = sys_lgetxattr(par_path, xattr_name, NULL, 0);
+        }
+        if (size < 0) {
+            op_ret = -1;
+            gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_PREOP_CHECK_FAILED,
+                   "getxattr on key (%s)  failed, path : %s", xattr_name,
+                   par_path);
+            goto out;
+        }
+    }
+
+    if (!have_val) {
+        size = sys_lgetxattr(par_path, xattr_name, value_buf, size);
+        if (size < 0) {
+            gf_msg(this->name, GF_LOG_ERROR, errno, P_MSG_PREOP_CHECK_FAILED,
+                   "getxattr on key (%s) failed (%s)", xattr_name,
+                   strerror(errno));
+            goto out;
+        }
+    }
+
+    if ((arg_data->len != size) || (memcmp(arg_data->data, value_buf, size))) {
+        gf_msg(this->name, GF_LOG_INFO, EIO, P_MSG_PREOP_CHECK_FAILED,
+               "failing preop as on-disk xattr value differs from argument "
+               "value for key %s",
+               xattr_name);
+        op_ret = -1;
+    }
+
+out:
+    dict_del_sizen(xdata, xattr_name);
+    dict_del_sizen(xdata, GF_PREOP_PARENT_KEY);
+
+    if (op_ret == -1) {
+        is_stale = _gf_true;
+    }
+
+    return is_stale;
+}
